% End to end model - PIC model

% Code for my Master's Thesis: On the Cell-to-Cell Extracellular Vesicle Transmission within the Brain
% Case ﻿Study: Alzheimer's Disease
% Author: Dorsa Kabiri
% Date: 07/10/2023
% Supervisors: Mladen Veletić, Martin Damrath, Ilangko Balasingham,
% Norwegian University of Science and Technology


% Transmitter part is from the solution in: 
% M. Veletić, M. T. Barros, H. Arjmandi, S. Balasubramaniam, and I. Balasingham, “Modeling
% of Modulated Exosome Release From Differentiated Induced Neural Stem Cells for Targeted
% Drug Delivery,” IEEE Transactions on NanoBioscience, vol. 19, no. 3, pp. 357–367, 7 2020


clc
clear all;

%% Defining the time vector
step = 1;                   %[ms]
step_sec = step/1000;       %[s]
t_end = 10;%1e-3; %8;
t_start = 1e-12;
num_samples = (t_end - t_start)/step_sec;   
tvec = linspace( 1e-12, t_end, num_samples);       %[s]

npulse = 1;
time = 01e-012:step:1e3; %[ms]
amp = 40;          %[\muA/cm^2; pA]
offset = 50;

%% Parameter Set: ---
% NEURONAL CELLS
% Nominal values for electrical activity
% Voltages given in [mV]
% Conductances given in [nS or, equivalently, mS/cm^3]
% Time constants given in [ms]
% Capacitance given in [\muF/cm^2]
gK = 36;
gNa = 120;
gL = 0.3;
VNa = 50;
VK = -70;
VL = -54.4;
cm = 1;

VCa = 65;
VSOC = 0;
gSOC = 0;
gCaL = 0.7;
gCaPQ = 0.6;
gCaT = 0.4;
gKa = 1;
gKATP = 0.295;
VmCaL = -30;
VmCaPQ = -5;
SmCaL = 10;
SmCaPQ = 10;
VhCaL = -33;
VhCaPQ = -33;
ShCaL = -5;
ShCaPQ = -5;
taumVCaL = 1;
taumVCaPQ = 1;
taum0CaL = 0.05;
taum0CaPQ = 0.05;
VtaumCaL = -23;
VtaumCaPQ = -23;
StaumCaL = 20;
StaumCaPQ = 20;
tauhVCaL = 60;
tauhVCaPQ = 60;
tauh0CaL = 51;
tauh0CaPQ = 51;
VtauhCaL = 0;
VtauhCaPQ = 0;
StauhCaL = 20;
StauhCaPQ = 20;
VmCaT = -49;
SmCaT = 4;
VhCaT = -52;
ShCaT = -5;
taumVCaT = 15;
taum0CaT = 0;
VtaumCaT = -50;
StaumCaT = 12;
tauhVCaT = 20;
tauh0CaT = 5;
VtauhCaT = -50;
StauhCaT = 15;

VmNa = -30;
SmNa = 4;
VhNa = -52;
ShNa = -8;
taumVNa = 6;
taum0Na = 0.05;
VtaumNa = -50;
StaumNa = 10;
tauhVNa = 120;
tauh0Na = 0.5;
VtauhNa = -50;
StauhNa = 8;
VmKa = -45;
SmKa = 10;
VhKa = -68;
ShKa = -10;
taumVKa = 0;
taum0Ka = 0.1;
tauhVKa = 60;
tauh0Ka = 5;
VtauhKa = 5;
StauhKa = 20;
VmK = -25;
SmK = 23;
taumVK = 1.5;
taum0K = 15;
VtaumK = -10;
StaumK = 25;

f = 0.01;
Volmud = 2.618e-19; % [L]
Volc = 5.725e-13;   % [L]
NPQ = 0.0001;
Bmud = 264;         % [1/ms]
kPMCA = 0.3;        % [1/ms]
pleak = 3e-4;       % [1/ms]
nPQ = 4;
nL = 4;
nm = 4;
alpha = 5.18e-15;   % [\mumol/pA/ms]
Volm = 5.149e-14;   % [L]
VolcDIVVoler = 31;
NL = 200;
Bm = 0.128;         % [1/ms]
kSERCA = 0.1;       % [1/ms]
KPQ = 2;
KL = 50;            % [muM]
Km = 2;             % [muM]


%% Derivatives
% x(1) = Vm;
% x(2) = mCaL;
% x(3) = mCaPQ;
% x(4) = mCaT;
% x(5) = mNa;   -->m
% x(6) = mK;    -->n
% x(7) = mKa;
% x(8) = hCaL;
% x(9) = hCaPQ;
% x(10) = hCaT;
% x(11) = hNa;  -->h
% x(12) = hKa;
% x(13) = CaL;
% x(14) = CaPQ;
% x(15) = Cam;
% x(16) = Cac;
% x(17) = Caer;
%-----------------2---------------------------------------------------------
f1 = @(t,x) [
    -(gK*x(6)^4*(x(1)-VK) + gNa*x(5)^3*x(11)*(x(1)-VNa) + gL*(x(1)-VL)- controlSignal(t,amp,time,step,npulse, offset))/cm;
    (funcminf(x(1), VmCaL, SmCaL) - x(2))./functaum(x(1), taumVCaL, VtaumCaL, StaumCaL, taum0CaL);
    (funcminf(x(1), VmCaPQ, SmCaPQ) - x(3))./functaum(x(1), taumVCaPQ, VtaumCaPQ, StaumCaPQ, taum0CaPQ);
    (funcminf(x(1), VmCaT, SmCaT) - x(4))./functaum(x(1), taumVCaT, VtaumCaT, StaumCaT, taum0CaT);
    0.1*(x(1)+40)/(1-exp(-(x(1)+40)/10))*(1-x(5))-4*exp(-(x(1)+65)/18)*x(5);
    0.01*(x(1)+55)/(1-exp(-(x(1)+55)/10))*(1-x(6))-0.125*exp(-(x(1)+65)/80)*x(6);
    (funcminf(x(1), VmKa, SmKa) - x(7))./functaum(x(1), taumVKa, VtaumK, StaumK, taum0Ka);
    (funcminf(x(1), VhCaL, ShCaL) - x(8))./functaum(x(1), tauhVCaL, VtauhCaL, StauhCaL, tauh0CaL);
    (funcminf(x(1), VhCaPQ, ShCaPQ) - x(9))./functaum(x(1), tauhVCaPQ, VtauhCaPQ, StauhCaPQ, tauh0CaPQ);
    (funcminf(x(1), VhCaT, ShCaT) - x(10))./functaum(x(1), tauhVCaT, VtauhCaT, StauhCaT, tauh0CaT);
    0.07*exp(-(x(1)+65)/20)*(1-x(11))-1/(1+exp(-(x(1)+35)/10))*x(11);
    (funcminf(x(1), VhKa, ShKa) - x(12))./functaum(x(1), tauhVKa, VtauhKa, StauhKa, tauh0Ka);
    (-f*alpha*gCaL.*(x(1)-VCa)./NL./Volmud - f*Bmud*(x(13)-x(15)));
    (-f*alpha*gCaPQ.*(x(1)-VCa)./NPQ./Volmud*0 - f*Bmud*(x(14)-x(15)));
    (f*(-alpha*(gCaT.*x(4)^3*x(10)*(x(1)-VCa))/Volm + NPQ*Volmud/Volm*Bmud*x(3)*x(9)*(x(14)-x(15)) + NL*Volmud/Volm*Bmud*x(2)^2*x(8)*(x(13)-x(15)) - Volc/Volm*kPMCA*x(15) + Volc/Volm*Bm*(x(15)-x(16))));
    (f*(Bm*(x(15)-x(16)) + pleak*(x(17)-x(16)) - kSERCA*x(16)));
    (-f*VolcDIVVoler*(pleak*(x(17)-x(16)) - kSERCA*x(16)))
    ];
% ODE with the initial condition
xinit = [-65 zeros(1,16)];
t_start = tic;
tspan_ode = [0 time(end)];
[t1, sol] = ode45(f1, time, xinit);
t_end = toc(t_start);
fprintf('\n     [ OK ]');
fprintf('\n     Elapsed %.4f s',t_end);


%% Control Signal & Plasma Membrane Potential
i_stimulation = controlSignal(t1,amp,time,step,npulse,offset);
Vm = sol(:, 1);
figure,
subplot(2,2,[1,2]), plot(t1/1000,i_stimulation, 'LineWidth', 1); hold on; grid minor;
title('Control signal', 'Interpreter', 'latex', 'FontSize', 11);
xlabel('Time [s]', 'Interpreter', 'latex', 'FontSize', 11);
ylabel('$i_m$ [$\mu$A/cm$^2$]', 'Interpreter', 'latex', ...
    'FontSize', 11);
subplot(2,2,[3,4]), plot(t1/1000, Vm, 'LineWidth', 1); hold on; grid minor;
title('Plasma membrane potential', 'Interpreter', 'latex', 'FontSize', 11);
xlabel('Time [s]', 'Interpreter', 'latex', 'FontSize', 11);
ylabel('$v_m$ [mV]', 'Interpreter', 'latex', ...
    'FontSize', 11);

%% Activation and Inactivation Functions from ODEs
% Activation Functions
mCaL = sol(:,2);
mCaPQ = sol(:,3);
mCaT = sol(:,4);
mNa = sol(:,5);
mK = sol(:,6);
mKa = sol(:,7);

% % Inactivation Functions
hCaL = sol(:,8);
hCaPQ = sol(:,9);
hCaT = sol(:,10);
hNa = sol(:,11);
hKa = sol(:,12);


%% Calcium Concentrations from ODEs
CaLo = sol(:,13);
CaPQo = sol(:,14);
Cam = sol(:,15);
Cac = sol(:,16);
Caer = sol(:,17);

figure,
subplot(1,2,1), plot(t1/1000, CaLo, 'LineWidth', 1); hold on; grid minor;
title('$[\mathrm{Ca}]_{L}$', 'Interpreter', 'latex', 'FontSize', 11);
xlabel('Time [s]', 'Interpreter', 'latex', 'FontSize', 11);
ylabel('$\mu$mol/L', 'Interpreter', 'latex', 'FontSize', 11);
subplot(1,2,2), plot(t1/1000, Cam, 'LineWidth', 1); hold on; grid minor;
title('$[\mathrm{Ca}]_m$', 'Interpreter', 'latex', 'FontSize', 11);
xlabel('Time [s]', 'Interpreter', 'latex', 'FontSize', 11);
ylabel('$\mu$mol/L', 'Interpreter', 'latex', 'FontSize', 11);


%% Exocytosis Equations
% Release Rates
rateCaL = mCaL.^2.*hCaL.*(CaLo.^nL)./(CaLo.^nL + KL.^nL) + ...
    (1 - mCaL.^2.*hCaL).*(Cam.^nL)./(Cam.^nL + KL.^nL);
rateCam = (Cam.^Km)./(Cam.^nm + Km.^nm);

figure,
plot(t1/1000, (rateCaL+rateCam)/(6.022e014), 'LineWidth', 2); hold on; grid minor;
xlabel('Time [s]', 'Interpreter', 'latex', 'FontSize', 11);
ylabel('$\mathrm{particle}$/L/ms', 'Interpreter', 'latex', 'FontSize', 11);
ylim([0, 8e-17]);
title('$\mathcal{R}^{\mathrm{(neuron)}}$',  sprintf('$g_{Na}=%g$ ms/${cm}^3$', gNa) , 'Interpreter', 'latex', 'FontSize', 11);

%% Released Concentrations
cExoL = zeros(size(rateCaL));
cExom = zeros(size(rateCam));
for i = 1:length(rateCaL)
    cExoL(i) = trapz(rateCaL(1:i))*step;
    cExom(i) = trapz(rateCam(1:i))*step;
end



%% PIC probabilites
rvec = [2e-06 3e-6 4e-06 5e-06];  % position vector [m]

% pic parameters
D = 1e-012;%5e-9;%1e-12;   % diffusion coefficient
lambda = 1.6 ;  %  for brain extracellular matrix 
K = D/(lambda^2);
r0 = 1e-06;  % release point
a = 0.5e-06; % receiver radius
alpha = 0.2;
kd = 2e-04;%2e03  % degradation rate
Nref = 1;    % reference number of particles
kf = 3.14e-14; %forward rate
ki = 1;%1e4;      % internalization rate
kb = 0.11;%2e3;   % backward rate
N_PBS = 5000; % number_of _particles
R_PBS = 1000; % number of realizations

sol = solvewolfram_conf(kf, kb, ki, kd, K, a);

al = sol(1);  %alpha
b =  sol(2);   %beta
g =  sol(3);   %gamma
        
et1 = al*(g+al)*(al+b)./((g-al)*(al-b));
et2 = b*(g+b)*(al+b)./((b-g)*(al-b));
et3 = g*(g+b)*(al+g)./((b-g)*(g-al));


% The probability p(r, t|r0) thata given particle released
% at distance r0 at time t0 = 0 is at
% position r at time t, 

p_a = @(t,r) exp(-kd.*t).*(1./(8.*pi.*r.*r0.*sqrt(pi.*t.*K)).*...
    (exp(-(r-r0).^2./(4.*t.*K)) + exp(-(r+r0-2.*a).^2./(4.*t.*K))) + ...
    1./(4.*pi.*r.*r0.*sqrt(K)).*real(...
    et1.*W_func((r+r0-2.*a)./sqrt(4.*t.*K),sol(1).*sqrt(t)) + ...
    et2.*W_func((r+r0-2.*a)./sqrt(4.*t.*K),sol(2).*sqrt(t)) + ...
    et3.*W_func((r+r0-2.*a)./sqrt(4.*t.*K),sol(3).*sqrt(t))));

P_f = @(t) kf*Nref./(4*pi*sqrt(K)*r0*a).*exp(-kd*t).*real(...
    al*W_func((r0-a)./(2*sqrt(K*t)), al*sqrt(t))/((al-b)*(g-al)) + ...
    b*W_func((r0-a)./(2*sqrt(K*t)), b*sqrt(t))/((b-g)*(al-b)) + ...
    g*W_func((r0-a)./(2*sqrt(K*t)), g*sqrt(t))/((b-g)*(g-al)));


P_e = @(t) integral(@(r) (4*pi*r.^2).*p_a(t,r), a, 20*sqrt(K*t_end));

Pi_inf = ki*kf*Nref/(4*pi*r0*a*sqrt(K*kd))*exp(-sqrt(kd/K)*(r0-a))*...
    [al/((g-al)*(al-b)*(sqrt(kd)+al)) + b/((b-g)*(al-b)*(sqrt(kd)+b)) + g/((b-g)*(g-al)*(sqrt(kd)+g))];

Prob_f = P_f(tvec);
Prob_i = ki*cumtrapz(tvec, Prob_f);


Prob_e = zeros(1, length(tvec));


p_a_plot = zeros(length(tvec), length(rvec));
for n = 1:length(tvec)
   for m = 1:length(rvec)
       p_a_plot(n, m) = p_a(tvec(n), rvec(m));
   end  
   Prob_e(n) = P_e(tvec(n));
end

Prob_d = kd*cumtrapz(tvec,Prob_e);


%% Channel responce
%channel input
released_vec = (rateCam+rateCaL)/(6.022e014)*step;

%channel output - bound EVs
conv1 = conv(Prob_f, released_vec);
bound_EVs = conv1(1:length(Prob_f));


%channel output - detected EVs
conv2 = conv(Prob_i, released_vec);
detected_EVs = conv2(1:length(Prob_f));


figure
plot(tvec, p_a_plot); grid minor;
title('Probility function of EV concentration', sprintf('$\\lambda$=$%g$ , $g_{Na}=%g$ ms/${cm}^3$', double(lambda), gNa), 'Interpreter', 'latex', 'FontSize', 11)
xlabel('Time [s]', 'Interpreter', 'latex', 'FontSize', 11);
legend('$r = $' + string(rvec*1e06) + ' $\mathrm{\mu m}$', 'Interpreter', 'latex', 'location', 'southeast')
ylabel('$p(r, t|r_0)$', 'Interpreter', 'latex', 'FontSize', 11);




figure
plot(tvec, detected_EVs); grid minor;
title('Internalized EVs', sprintf('$\\lambda$=$%g$ , $g_{Na}=%g$ ms/${cm}^3$', double(lambda), gNa), 'Interpreter', 'latex', 'FontSize', 11)
xlabel('Time [s]', 'Interpreter', 'latex', 'FontSize', 11);
ylabel('[particle/L]', 'Interpreter', 'latex', 'FontSize', 11);



figure
plot(tvec, bound_EVs); grid minor;
title('Bound EVs', sprintf('$\\lambda$=$%g$ , $g_{Na}=%g$ ms/${cm}^3$', double(lambda), gNa), 'Interpreter', 'latex', 'FontSize', 11)
xlabel('Time [s]', 'Interpreter', 'latex', 'FontSize', 11);
ylabel('[particle/L]', 'Interpreter', 'latex', 'FontSize', 11);
